"""Base email backend class."""


class BaseEmailBackend(object):
    """
    Base class for email backend implementations.

    Subclasses must at least overwrite send_messages().
    """
    def __init__(self, fail_silently=False, **kwargs):
        self.fail_silently = fail_silently

    def open(self):
        """Open a network connection.

        This method can be overwritten by backend implementations to
        open a network connection.

        It's up to the backend implementation to track the status of
        a network connection if it's needed by the backend.

        This method can be called by applications to force a single
        network connection to be used when sending mails. See the
        send_messages() method of the SMTP backend for a reference
        implementation.

        The default implementation does nothing.
        """
        pass

    def close(self):
        """Close a network connection."""
        pass

    def send_messages(self, email_messages):
        """
        Sends one or more EmailMessage objects and returns the number of email
        messages sent.
        """
        num_sent = 0
        for email_message in email_messages:
            if self._send_message_wrapper(email_message):
                num_sent += 1
        return num_sent

    def _send_message_wrapper(self, email_message):
        """A helper method that does the actual sending."""

        if not email_message.recipients():
            return False

        try:
            self._send_message(email_message)
        except:
            from beproud.django.mailer.api import log_exception
            log_exception("%s: Mail Error" % self)
            if not self.fail_silently:
                raise
            return False
        return True

    def _send_message(self, email_message):
        raise NotImplementedError



#:coding=utf-8:
import sys
import traceback
import logging
import six
from six.moves import email_mime_base

from email import encoders, charset, message_from_string
from email.utils import formatdate
from email.message import Message
from email.mime.text import MIMEText
from email.mime.message import MIMEMessage

from django.core import mail as django_mail
from django.template.loader import render_to_string
from django.conf import settings

from beproud.django.mailer.signals import mail_pre_send, mail_post_send

# NOTE: CHARSETSや、ALIASESを先に登録しておかないといけない
from beproud.django.mailer.models import *  # NOQA

__version__ = '0.35'

__all__ = (
    'get_connection',
    'forbid_multi_line_headers',
    'make_msgid',
    'send_mail',
    'send_template_mail',
    'send_mass_mail',
    'mail_managers',
    'mail_managers_template',
    'mail_admins',
    'render_message',
    'EmailMessage',
    'EmailMultiAlternatives',
    'SafeMIMEMessage',
    'SafeMIMEText',
    'SafeMIMEMultipart',
    'BadHeaderError',
)

StringIO = six.StringIO
MIMEBase = email_mime_base.MIMEBase

utf8_charset = charset.Charset('utf-8')

# ガラケの場合は base64 じゃないとダメなやつが多いので、デフォルトで BASE64を使う。
if not getattr(settings, "EMAIL_USE_BASE64_FOR_UTF8", True):
    # Don't BASE64-encode UTF-8 messages so that we avoid unwanted attention from
    # some spam filters.
    utf8_charset.body_encoding = None


class SafeMIMEText(django_mail.SafeMIMEText):
    def __init__(self, text, subtype, charset):
        self.encoding = charset
        # NOTE: utf8 でも utf-8 でも対応する
        if charset.upper() in  ('UTF8', 'UTF-8'):
            # Unfortunately, Python doesn't support setting a Charset instance
            # as MIMEText init parameter (http://bugs.python.org/issue16324).
            # We do it manually and trigger re-encoding of the payload.
            MIMEText.__init__(self, text, subtype, None)
            del self['Content-Transfer-Encoding']
            self.set_payload(text, utf8_charset)
            self.replace_header('Content-Type', 'text/%s; charset="%s"'
                                % (subtype, utf8_charset.get_output_charset()))
        else:
            MIMEText.__init__(self, text, subtype, charset)


django_mail.SafeMIMEText = SafeMIMEText
try:
    from django.core.mail import message as django_mail_message
    django_mail_message.SafeMIMEText = SafeMIMEText
except ImportError:
    pass


SafeMIMEMultipart = django_mail.SafeMIMEMultipart
BadHeaderError = django_mail.BadHeaderError
get_connection = django_mail.get_connection
forbid_multi_line_headers = django_mail.forbid_multi_line_headers
make_msgid = django_mail.make_msgid


class SafeMIMEMessage(MIMEMessage):
    def __setitem__(self, name, val):
        # message/rfc822 attachments must be ASCII
        name, val = forbid_multi_line_headers(name, val, 'ascii')
        MIMEMessage.__setitem__(self, name, val)


logger = logging.getLogger(getattr(settings, "EMAIL_LOGGER", ""))


class EmailMessage(django_mail.EmailMessage):
    def message(self):
        encoding = self.encoding or getattr(settings, "EMAIL_CHARSET", settings.DEFAULT_CHARSET)
        msg = SafeMIMEText(self.body, self.content_subtype, encoding)
        msg = self._create_message(msg)
        msg['Subject'] = self.subject
        msg['From'] = self.extra_headers.get('From', self.from_email)
        msg['To'] = ', '.join(self.to)
        if self.cc:
            msg['Cc'] = ', '.join(self.cc)

        # Email header names are case-insensitive (RFC 2045), so we have to
        # accommodate that when doing comparisons.
        header_names = [key.lower() for key in self.extra_headers]
        if 'date' not in header_names:
            msg['Date'] = formatdate(localtime=getattr(settings, "EMAIL_USE_LOCALTIME", False))
        if 'message-id' not in header_names:
            msg['Message-ID'] = make_msgid()
        for name, value in self.extra_headers.items():
            if name.lower() == 'from':  # From is already handled
                continue
            msg[name] = value
        return msg

    def _create_mime_attachment(self, content, mimetype):
        """
        Converts the content, mimetype pair into a MIME attachment object.

        If the mimetype is message/rfc822, content may be an
        email.Message or EmailMessage object, as well as a str.
        """
        basetype, subtype = mimetype.split('/', 1)
        if basetype == 'text':
            encoding = self.encoding or getattr(settings, "EMAIL_CHARSET",
                                                settings.DEFAULT_CHARSET)
            attachment = SafeMIMEText(content, subtype, encoding)
        elif basetype == 'message' and subtype == 'rfc822':
            # Bug #18967: per RFC2046 s5.2.1, message/rfc822 attachments
            # must not be base64 encoded.
            if isinstance(content, EmailMessage):
                # convert content into an email.Message first
                content = content.message()
            elif not isinstance(content, Message):
                # For compatibility with existing code, parse the message
                # into an email.Message object if it is not one already.
                content = message_from_string(content)

            attachment = SafeMIMEMessage(content, subtype)
        else:
            # Encode non-text attachments with base64.
            attachment = MIMEBase(basetype, subtype)
            attachment.set_payload(content)
            encoders.encode_base64(attachment)
        return attachment

    def send(self, *args, **kwargs):
        mail_pre_send.send(sender=self, message=self)
        super(EmailMessage, self).send(*args, **kwargs)
        mail_post_send.send(sender=self, message=self)


class EmailMultiAlternatives(EmailMessage):
    """
    A version of EmailMessage that makes it easy to send multipart/alternative
    messages. For example, including text and HTML versions of the text is
    made easier.
    """
    alternative_subtype = 'alternative'

    def __init__(self, subject='', body='', from_email=None, to=None, bcc=None,
                 connection=None, attachments=None, headers=None, alternatives=None,
                 cc=None):
        """
        Initialize a single email message (which can be sent to multiple
        recipients).

        All strings used to create the message can be unicode strings (or UTF-8
        bytestrings). The SafeMIMEText class will handle any necessary encoding
        conversions.
        """
        super(EmailMultiAlternatives, self).__init__(subject, body, from_email, to, bcc,
                                                     connection, attachments, headers, cc)
        self.alternatives = alternatives or []

    def attach_alternative(self, content, mimetype):
        """Attach an alternative content representation."""
        assert content is not None
        assert mimetype is not None
        self.alternatives.append((content, mimetype))

    def _create_message(self, msg):
        return self._create_attachments(self._create_alternatives(msg))

    def _create_alternatives(self, msg):
        encoding = self.encoding or getattr(settings, "EMAIL_CHARSET", settings.DEFAULT_CHARSET)
        if self.alternatives:
            body_msg = msg
            msg = SafeMIMEMultipart(_subtype=self.alternative_subtype, encoding=encoding)
            if self.body:
                msg.attach(body_msg)
            for alternative in self.alternatives:
                msg.attach(self._create_mime_attachment(*alternative))
        return msg


def send_mail(subject, message, from_email, recipient_list,
              fail_silently=False, auth_user=None, auth_password=None, encoding=None,
              connection=None, html_message=None, cc=None, bcc=None, attachments=None):
    """
    Sends an email message.

    Arguments
    --------------

    * subject           -- The email subject
    * message           -- The email message body text
    * from_email        -- The sender. Can be an email address or formatted name and address.
                           Example: Ian Lewis <ian@example.com>
                           Example: ian@example.com
    * recipient_list    -- A list of recipients. This is formatted the same as the from_email.

    Keyword Arguments
    ----------------------
    fail_silently       -- A boolean indicating whether errors when sending emails should be
                           squelched. If True errors are squelched and logged. If False errors
                           are raised as Exceptions as normal. Default is False.
    auth_user           -- The username to use when authenticating with the SMTP server.
                           This argument is ignored if a connection object is provided.
                           Defaults to the EMAIL_HOST_USER in settings.py
    auth_password       -- The password to use when authenticating with the SMTP server.
                           This argument is ignored if a connection object is provided.
                           Defaults to the EMAIL_HOST_PASSWORD in settings.py
    encoding            -- The character encoding to use in the email.
                           Defaults to the EMAIL_CHARSET or DEFAULT_CHARSET in settings.py
    connection          -- The connection object to use when sending the email as returned
                           by get_connection()
    html_message        -- The html body part of the email. If provided the email is encoded
                           as a multi-part email with an html part containing the html body.
                           Useful for sending html emails.
    cc                  -- The cc email recipient list.
    bcc                 -- The bcc recipient list.
    attachments         -- A list of two/three tuples used as attachments for the email.
                           The fields in the tuple are as follows:

                               - A filename
                               - A str object containing the file content.
                               - The mimetype for the file (optional but recommended.)
                                 If the mimetype is not provided it is guessed.
    """

    if settings.DEBUG and hasattr(settings, "EMAIL_ALL_FORWARD"):
        recipient_list = [settings.EMAIL_ALL_FORWARD]
        from_email = settings.EMAIL_ALL_FORWARD

    connection = connection or get_connection(username=auth_user, password=auth_password,
                                              fail_silently=fail_silently)
    if html_message:
        msg = EmailMultiAlternatives(
            subject=subject,
            body=message,
            from_email=from_email,
            to=recipient_list,
            cc=cc,
            bcc=bcc,
            connection=connection,
        )
        msg.attach_alternative(html_message, "text/html")
    else:
        msg = EmailMessage(
            subject=subject,
            body=message,
            from_email=from_email,
            to=recipient_list,
            cc=cc,
            bcc=bcc,
            connection=connection,
        )

    if attachments:
        for attachment in attachments:
            msg.attach(*attachment)

    msg.encoding = encoding
    return_val = msg.send()
    log_message(msg, return_val)
    return return_val


def _render_mail_template(template_name, extra_context=None):
    """
    Renders the template and returns the resulting text.
    """
    context = {}
    context.update(getattr(settings, "EMAIL_DEFAULT_CONTEXT", {}))
    context.update(extra_context or {})
    return render_to_string(template_name, context)


def render_message(template_name, extra_context={}):
    """
    Renders a text email message from a template and returns a two
    tuple containing the subject and body of the message.

    The contents of the EMAIL_DEFAULT_CONTENT setting are
    passed to the template when it is rendered but can be
    overridden.
    """
    mail_text = _render_mail_template(template_name, extra_context)
    rendered_mail = mail_text.replace(u"\r\n", u"\n").replace(u"\r", u"\n").split(u"\n")
    return rendered_mail[0], "\n".join(rendered_mail[1:])


def send_template_mail(template_name, from_email, recipient_list, extra_context={},
                       fail_silently=False, auth_user=None, auth_password=None, encoding=None,
                       connection=None, html_template_name=None, cc=None, bcc=None,
                       attachments=None):
    u"""
    Send an email using a django template. The template should be formatted
    so that the first line of the template is the subject. All subsequent lines
    are used as the body of the email message. The easiest way to create a template is to
    extend the "mail.tpl" template provided with bpmailer and specify the subject and body
    blocks.

    You can also specify the "mail.tpl" template and add the "subject" and "body" to
    the extra_context argument to use bpmailer's default template.

    Arguments
    --------------

    * template_name     -- The name of the Django template to use to render the email.
    * from_email        -- The sender. Can be an email address or formatted name and address.
                           Example: Ian Lewis <ian@example.com>
                           Example: ian@example.com
    * recipient_list    -- A list of recipients. This is formatted the same as the from_email.
    * extra_context     -- An dictionary of extra data that is added to the context when
                           rendering the template.

    Keyword Arguments
    ----------------------
    fail_silently       -- A boolean indicating whether errors when sending emails should be
                           squelched. If True errors are squelched and logged. If False errors
                           are raised as Exceptions as normal. Default is False.
    auth_user           -- The username to use when authenticating with the SMTP server.
                           This argument is ignored if a connection object is provided.
                           Defaults to the EMAIL_HOST_USER in settings.py
    auth_password       -- The password to use when authenticating with the SMTP server.
                           This argument is ignored if a connection object is provided.
                           Defaults to the EMAIL_HOST_PASSWORD in settings.py
    encoding            -- The character encoding to use in the email.
                           Defaults to the EMAIL_CHARSET or DEFAULT_CHARSET in settings.py
    connection          -- The connection object to use when sending the email as returned
                           by get_connection()
    html_template_name  -- The template for the html body part of the email. If provided
                           the email is encoded as a multi-part email with an html part
                           containing the html body.  Useful for sending html emails.
    cc                  -- The cc email recipient list.
    bcc                 -- The bcc recipient list.
    attachments         -- A list of two/three tuples used as attachments for the email.
                           The fields in the tuple are as follows:

                               - A filename
                               - A str object containing the file content.
                               - The mimetype for the file (optional but recommended.)
                                 If the mimetype is not provided it is guessed.
    """
    if not isinstance(recipient_list, list) and not isinstance(recipient_list, tuple):
        recipient_list = [recipient_list]

    html_message = None
    try:
        subject, message = render_message(template_name, extra_context)
        if html_template_name:
            html_message = _render_mail_template(html_template_name, extra_context)
    except Exception:
        log_exception("Mail Error")
        if fail_silently:
            return
        else:
            raise

    return send_mail(
        subject=subject,
        message=message,
        recipient_list=recipient_list,
        cc=cc,
        bcc=bcc,
        from_email=from_email,
        fail_silently=fail_silently,
        auth_user=auth_user,
        auth_password=auth_password,
        encoding=encoding,
        connection=connection,
        html_message=html_message,
        attachments=attachments,
    )


def send_mass_mail(datatuple, fail_silently=False, auth_user=None,
                   auth_password=None, encoding=None, connection=None):
    """
    Given a datatuple of (subject, message, from_email, recipient_list), sends
    each message to each recipient list. Returns the number of e-mails sent.
    Also supports an optional encoding parameter to the datatuple. Individual
    items in the datatuple may have a fifth item which specifies the encoding
    of the particular email. Encodings in the datatuple have priority over
    the encoding passed to send_mass_mail.

    If from_email is None, the DEFAULT_FROM_EMAIL setting is used.
    If auth_user and auth_password are set, they're used to log in.
    If auth_user is None, the EMAIL_HOST_USER setting is used.
    If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.

    Note: The API for this method is frozen. New code wanting to extend the
    functionality should use the EmailMessage class directly.
    """
    connection = connection or get_connection(username=auth_user, password=auth_password,
                                              fail_silently=fail_silently)

    def _message(args):
        if isinstance(args, EmailMessage):
            return args
        if len(args) > 4:
            subject, body, sender, recipient, charset = args
        else:
            subject, body, sender, recipient = args
            charset = encoding or None
        message = EmailMessage(
            subject=subject,
            body=body,
            from_email=sender,
            to=recipient,
            connection=connection,
        )
        if charset:
            message.encoding = charset
        return message

    messages = [_message(d) for d in datatuple]

    for message in messages:
        mail_pre_send.send(sender=message, message=message)
    return_val = connection.send_messages(messages)
    for message in messages:
        mail_post_send.send(sender=message, message=message)
    return return_val


def mail_managers(subject, message, fail_silently=False, encoding=None):
    if not settings.MANAGERS:
        return
    send_mail(
        subject=settings.EMAIL_SUBJECT_PREFIX + subject,
        message=message,
        from_email=settings.SERVER_EMAIL,
        recipient_list=[a[1] for a in settings.MANAGERS],
        fail_silently=fail_silently,
        encoding=encoding,
    )


def mail_managers_template(template_name, extra_context={}, fail_silently=False, encoding=None):
    if not settings.MANAGERS:
        return
    return send_template_mail(
        template_name=template_name,
        recipient_list=[a[1] for a in settings.MANAGERS],
        extra_context=extra_context,
        from_email=settings.SERVER_EMAIL,
        fail_silently=fail_silently,
        encoding=encoding,
    )


def mail_admins(subject, message, fail_silently=False, encoding=None):
    if not settings.ADMINS:
        return
    send_mail(
        subject=settings.EMAIL_SUBJECT_PREFIX + subject,
        message=message,
        from_email=settings.SERVER_EMAIL,
        recipient_list=[a[1] for a in settings.ADMINS],
        fail_silently=fail_silently,
        encoding=encoding,
    )


def log_message(msg, sent):
    message = "%s messages sent.\n" % sent
    message += "From: %s\n" % msg.from_email
    message += "To: %s\n" % ", ".join(msg.to)
    message += "Subject: %s\n" % msg.subject
    message += "Body\n"
    message += "-"*30 + "\n"
    message += msg.body

    logger.info(message)


def log_exception(msg=""):
    msg = msg+"\n" if msg else ""
    tb = ''.join(traceback.format_exception(sys.exc_info()[0],
                 sys.exc_info()[1], sys.exc_info()[2]))
    logger.exception(msg + tb)



"""
Dummy email backend that does nothing.
"""

from beproud.django.mailer.backends.base import BaseEmailBackend


class EmailBackend(BaseEmailBackend):
    def _send_message(self, email_message):
        pass



#:coding=utf-8:
import six

from django.conf import settings

from email import charset
from email.charset import (
    add_alias, add_charset, add_codec,
    BASE64, SHORTEST,
)

__all__ = ('init_mailer',)


# Uppercase charset aliases cause inequality checks
# with input and output encodings to fail thus causing
# double encoding of base64 body text parts.
if six.PY2:
    def _safe_str(self):
        return self.input_charset
    charset.Charset.__str__ = _safe_str


# Python charset => mail header charset mapping
# TODO: Add more encodings
CHARSETS = getattr(settings, "EMAIL_CHARSETS", {
    'utf-8': {
        'header_enc': SHORTEST,
        'body_enc': None,
        'output_charset': None,
    },
    'shift-jis': {
        'header_enc': BASE64,
        'body_enc': None,
        'output_charset': None,
    },
    'iso-2022-jp': {
        'header_enc': BASE64,
        'body_enc': None,
        'output_charset': None,
    },
    'iso-2022-jp-2': {
        'header_enc': BASE64,
        'body_enc': None,
        'output_charset': None,
    },
    'iso-2022-jp-3': {
        'header_enc': BASE64,
        'body_enc': None,
        'output_charset': None,
    },
    'iso-2022-jp-ext': {
        'header_enc': BASE64,
        'body_enc': None,
        'output_charset': None,
    },
})

ALIASES = getattr(settings, "EMAIL_CHARSET_ALIASES", {
    # utf-8
    "utf8": "utf-8",
    "utf_8": "utf-8",
    "U8": "utf-8",
    "UTF": "utf-8",
    "utf8": "utf-8",
    "utf-8": "utf-8",

    # Shift-JIS
    "cp932": "shift-jis",
    "932": "shift-jis",
    "ms932": "shift-jis",
    "mskanji": "shift-jis",
    "ms-kanji": "shift-jis",

    "shift_jis": "shift-jis",
    "csshiftjis": "shift-jis",
    "shiftjis": "shift-jis",
    "sjis": "shift-jis",
    "s_jis": "shift-jis",

    #"shift_jis_2004": "shift-jis",
    #"shiftjis2004": "shift-jis",
    #"sjis_2004": "shift-jis",
    #"sjis2004": "shift-jis",
    #
    #"shift_jisx0213": "shift-jis",
    #"shiftjisx0213": "shift-jis",
    #"sjisx0213": "shift-jis",
    #"s_jisx0213": "shift-jis",

    # ISO-2022-JP
    "iso2022_jp": "iso-2022-jp",
    "scsiso2022jp": "iso-2022-jp",
    "iso2022jp": "iso-2022-jp",
    "iso-2022-jp": "iso-2022-jp",
    "iso-2022-jp": "iso-2022-jp",
    "iso-2022-jp-1": "iso-2022-jp",

    "iso-2022-jp-2": "iso-2022-jp-2",
    "iso2022_jp_2": "iso-2022-jp-2",
    "iso2022jp-2": "iso-2022-jp-2",
    "iso2022_jp_2004": "iso-2022-jp-2",
    "iso2022jp-2004": "iso-2022-jp-2",
    "iso-2022-jp-2004": "iso-2022-jp-2",

    "iso2022_jp_3": "iso-2022-jp-3",
    "iso2022jp-3": "iso-2022-jp-3",
    "iso-2022-jp-3": "iso-2022-jp-3",

    # TODO: 携帯は対応してないと
    "iso2022_jp_ext": "iso-2022-jp-ext",
    "iso2022jp-ext": "iso-2022-jp-ext",
    "iso-2022-jp-ext": "iso-2022-jp-ext",
})
CODECS = getattr(settings, "EMAIL_CHARSET_CODECS", {
    'iso-2022-jp': 'iso-2022-jp',
    'iso-2022-jp-2': 'iso-2022-jp-2',
    'iso-2022-jp-3': 'iso-2022-jp-3',
    'iso-2022-jp-ext': 'iso2022jp-ext',
    'utf-8': 'utf-8',
    'shift-jis': 'cp932',
})


def init_mailer():
    if CHARSETS:
        for canonical, charset_dict in six.iteritems(CHARSETS):
            add_charset(canonical, **charset_dict)

    if ALIASES:
        for alias, canonical in six.iteritems(ALIASES):
            add_alias(alias, canonical)

    if CODECS:
        for canonical, codec_name in six.iteritems(CODECS):
            add_codec(canonical, codec_name)


init_mailer()



#:coding=utf-8:

import os
import time
import logging
import unittest
from itertools import chain
from logging.handlers import BufferingHandler

import mock

from django.test import TestCase as DjangoTestCase
from django.test import override_settings
from django.core import mail as django_mail
from django.conf import settings

from beproud.django.mailer import api as mailer_api
from beproud.django.mailer import tasks as mailer_tasks
from beproud.django.mailer.models import init_mailer
from beproud.django.mailer.backends.base import BaseEmailBackend
from beproud.django.mailer.signals import mail_pre_send, mail_post_send
from beproud.django.mailer import (
    EmailMessage,
    EmailMultiAlternatives,
    send_mail,
    send_template_mail,
    mail_managers,
    mail_admins,
    mail_managers_template,
    send_mass_mail,
)

# Suppress logging
logging.getLogger("").handlers = [BufferingHandler(0)]

__all__ = (
    'EncodingTestCaseUTF8',
    'EncodingTestCaseISO2022JP',
    'EmailAllForwardTestCase',
    'EmailAllForwardTestCase2',
    'TemplateTestCase',
    'TemplateContextTestCase',
    'DjangoMailISO2022JPTestCase',
    'DjangoMailUTF8TestCase',
    'SignalTest',
    'MassMailTest',
    'UTCTimeTestCase',
    'LocalTimeTestCase',
    'FailSilentlyTestCase',
    'AttachmentTestCase',
    'HtmlMailTestCase',

    'TaskTests',
)


class EmailError(Exception):
    pass


class ErrorEmailBackend(BaseEmailBackend):
    def _send_message(self, email_message):
        raise EmailError(u"ERROR")


class MailTestCase(object):
    def setUp(self):
        os.environ['TZ'] = settings.TIME_ZONE
        time.tzset()
        init_mailer()

    def tearDown(self):
        mail_pre_send.recievers = []
        mail_post_send.recievers = []


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class EmailMessageEncodingTestCase(MailTestCase, DjangoTestCase):
    def test_email_message_default(self):
        message_obj = EmailMessage(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example4@example.net>']
            )

        message = message_obj.message()

        self.assertEqual(message['Subject'], '=?utf-8?b?5Lu25ZCN?=')

    def test_email_message_utf_8(self):
        message_obj = EmailMessage(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example4@example.net>']
            )
        message_obj.encoding = 'utf-8'
        message = message_obj.message()

        self.assertEqual(message['Subject'], '=?utf-8?b?5Lu25ZCN?=')

    # FIXME
    @unittest.skip("Py3.6から動かない")
    def test_email_message_utf_8_alias(self):
        message_obj = EmailMessage(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example4@example.net>']
            )
        message_obj.encoding = 'UTF'
        message = message_obj.message()

        self.assertEqual(message['Subject'], '=?utf-8?b?5Lu25ZCN?=')

    # FIXME
    @unittest.skip("Py3.6から動かない")
    def test_email_message_cp932_alias_sjis(self):
        message_obj = EmailMessage(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example4@example.net>'],
            )
        message_obj.encoding = 'cp932'
        message = message_obj.message()

        self.assertEquals(message['Subject'], '=?shift-jis?b?jI+WvA==?=')

    def test_email_message_iso_2022_jp(self):
        message_obj = EmailMessage(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example4@example.net>'],
            )
        message_obj.encoding = 'iso-2022-jp'
        message = message_obj.message()

        self.assertEquals(message['Subject'], '=?iso-2022-jp?b?GyRCN29MPhsoQg==?=')

    def test_email_message_iso_2022_jp_alias(self):
        message_obj = EmailMessage(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example4@example.net>'],
            )
        message_obj.encoding = 'iso2022jp'
        message = message_obj.message()

        self.assertEquals(message['Subject'], '=?iso-2022-jp?b?GyRCN29MPhsoQg==?=')


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class EncodingTestCaseUTF8(MailTestCase, DjangoTestCase):

    def test_send_mail(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
        )
        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')

    def test_email_charset_strict(self):
        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()

        self.assertEqual(str(message['Subject']), "=?utf-8?b?5Lu25ZCN?=")
        self.assertEqual(str(message['To']), "=?utf-8?b?5a6b5YWI?= <example@example.net>")
        self.assertEqual(str(message['From']),
                         "=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '8bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="utf-8"')
        self.assertEqual(message.get_payload(decode=True), u'本文'.encode('utf-8'))

    def test_cc(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
            cc=['cc@example.net'],
        )
        self.assertEquals(len(django_mail.outbox), 1)

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['Cc']), 'cc@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')

    def test_bcc(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
            bcc=['bcc@example.net'],
        )
        self.assertEquals(len(django_mail.outbox), 1)

        message = django_mail.outbox[0].message()

        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')

        # Bcc is not included in email headers
        self.assertEquals(message['Bcc'], None)

        self.assertTrue(u'bcc@example.net' in django_mail.outbox[0].bcc)


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_CHARSET='iso-2022-jp')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class EncodingTestCaseISO2022JP(MailTestCase, DjangoTestCase):
    # TODO: Set ALIASES and CODECS

    def test_send_mail_encoding(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
            encoding="utf-8",
        )
        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')

    def test_email_charset(self):
        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?iso-2022-jp?b?GyRCN29MPhsoQg==?=')
        self.assertEquals(str(message['To']),
                          '=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>')

    def test_email_charset_strict(self):
        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()

        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(DEBUG=True)
@override_settings(EMAIL_ALL_FORWARD='all-forward@example.net')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class EmailAllForwardTestCase(MailTestCase, DjangoTestCase):
    def test_email_all_forward(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
        )
        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), 'all-forward@example.net')
        self.assertEquals(str(message['From']), 'all-forward@example.net')


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(DEBUG=False)
@override_settings(EMAIL_ALL_FORWARD='all-forward@example.net')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class EmailAllForwardTestCase2(MailTestCase, DjangoTestCase):

    def test_email_all_forward(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
        )
        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class TemplateTestCase(MailTestCase, DjangoTestCase):
    # TODO: Set ALIASES and CODECS

    def test_template_mail(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'subject': u'件名',
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文\n')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), '=?utf-8?b?5a6b5YWI?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')

    def test_multi_line_subject(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'subject': u'これは\r改行\nの\nある\r\n件名',
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 1)

        mail_message = django_mail.outbox[0]
        self.assertEquals(mail_message.subject, u'これは改行のある件名')
        self.assertEquals(mail_message.body, u'本文\n')

        message = mail_message.message()
        self.assertEquals(str(message['Subject']),
                          '=?utf-8?b?44GT44KM44Gv5pS56KGM44Gu44GC44KL5Lu25ZCN?=')
        self.assertEquals(str(message['To']), '=?utf-8?b?5a6b5YWI?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')

    def test_cc(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'差出人 <example-from@example.net>'],
            cc=[u'宛先 <example@example.net>'],
            extra_context={
                'subject': u'これは\r改行\nの\nある\r\n件名',
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 1)

        mail_message = django_mail.outbox[0]

        message = mail_message.message()
        self.assertEquals(str(message['To']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')
        self.assertEquals(str(message['Cc']), '=?utf-8?b?5a6b5YWI?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')

    def test_bcc(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'差出人 <example-from@example.net>'],
            bcc=[u'宛先 <example@example.net>'],
            extra_context={
                'subject': u'これは\r改行\nの\nある\r\n件名',
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 1)

        mail_message = django_mail.outbox[0]

        message = mail_message.message()
        self.assertEquals(str(message['To']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')

        # Bcc is not included in email headers
        self.assertEquals(message['Bcc'], None)

        self.assertTrue(u'宛先 <example@example.net>' in django_mail.outbox[0].bcc)


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_DEFAULT_CONTEXT={"subject": u"件名"})
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class TemplateContextTestCase(MailTestCase, DjangoTestCase):

    def test_email_default_context(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文\n')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), '=?utf-8?b?5a6b5YWI?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')

    def test_email_default_context_override(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'subject': 'overwrite',
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 1)
        self.assertEquals(django_mail.outbox[0].body, u'本文\n')

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), 'overwrite')
        self.assertEquals(str(message['To']), '=?utf-8?b?5a6b5YWI?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')

    def test_email_default_context_overwrite(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'subject': 'overwrite',
                'body': u'本文',
            },
            fail_silently=False,
        )

        # Check to make sure we don't overwrite the data in
        # settings.EMAIL_DEFAULT_CONTEXT
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'body': u'本文',
            },
            fail_silently=False,
        )

        self.assertEquals(len(django_mail.outbox), 2)
        self.assertEquals(django_mail.outbox[1].body, u'本文\n')

        message = django_mail.outbox[1].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), '=?utf-8?b?5a6b5YWI?= <example@example.net>')
        self.assertEquals(str(message['From']),
                          '=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>')


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='iso-2022-jp')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class DjangoMailISO2022JPTestCase(MailTestCase, DjangoTestCase):

    def test_send_mail(self):
        django_mail.send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()

        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class DjangoMailUTF8TestCase(MailTestCase, DjangoTestCase):
    def test_send_mail(self):
        django_mail.send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()

        self.assertEqual(str(message['Subject']), "=?utf-8?b?5Lu25ZCN?=")
        self.assertEqual(str(message['To']), "=?utf-8?b?5a6b5YWI?= <example@example.net>")
        self.assertEqual(str(message['From']),
                         "=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '8bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="utf-8"')
        self.assertEqual(message.get_payload(decode=True), u'本文'.encode('utf-8'))


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(EMAIL_CHARSET='iso-2022-jp')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class SignalTest(MailTestCase, DjangoTestCase):

    def test_pre_send_signal(self):
        def pre_send_signal(sender, message, **kwargs):
            message.from_email = message.from_email.replace(u'\uff5e', u'\u301c')
            message.to = [x.replace(u'\uff5e', u'\u301c') for x in message.to]
            message.bcc = [x.replace(u'\uff5e', u'\u301c') for x in message.bcc]
            message.subject = message.subject.replace(u'\uff5e', u'\u301c')
            message.body = message.body.replace(u'\uff5e', u'\u301c')
        mail_pre_send.connect(pre_send_signal)

        send_mail(
            u'件名',
            u'本文～テスト',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()

        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8!A%F%9%H\x1b(B")

    def test_post_send_signal(self):
        test_list = []

        def post_send_signal(sender, message, **kwargs):
            self.assertEqual(message.subject, u'件名')
            self.assertEqual(message.body, u'本文')
            test_list.append("arrived")
        mail_post_send.connect(post_send_signal)

        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        self.assertTrue(test_list, ["arrived"])


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class MassMailTest(MailTestCase, DjangoTestCase):

    def test_mass_mail(self):
        send_mass_mail(((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)))

        for i in range(10):
            message = django_mail.outbox[i].message()

            self.assertEqual(str(message['Subject']), "=?utf-8?b?5Lu25ZCN?=")
            self.assertEqual(str(message['To']),
                             "=?utf-8?b?5a6b5YWI?= <example%s@example.net>" % i)
            self.assertEqual(str(message['From']),
                             "=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>")
            self.assertEqual(message['Content-Transfer-Encoding'], '8bit')
            self.assertEqual(message['Content-Type'], 'text/plain; charset="utf-8"')
            self.assertEqual(message.get_payload(decode=True), u'本文'.encode('utf-8'))

    def test_mass_mail_encoding(self):
        send_mass_mail(((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)), encoding='iso-2022-jp')

        for i in range(10):
            message = django_mail.outbox[i].message()

            self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
            self.assertEqual(str(message['To']),
                             "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example%s@example.net>" % i)
            self.assertEqual(str(message['From']),
                             "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
            self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
            self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
            self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")

    def test_mass_mail_encoding_inline(self):
        send_mass_mail((
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example0@example.net>'],
                'iso-2022-jp',
            ),
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example1@example.net>'],
            ),
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example2@example.net>'],
                'iso-2022-jp',
            )
        ))

        message = django_mail.outbox[0].message()

        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example0@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")

        message = django_mail.outbox[1].message()
        self.assertEqual(str(message['Subject']), "=?utf-8?b?5Lu25ZCN?=")
        self.assertEqual(str(message['To']), "=?utf-8?b?5a6b5YWI?= <example1@example.net>")
        self.assertEqual(str(message['From']),
                         "=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '8bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="utf-8"')
        self.assertEqual(message.get_payload(decode=True), u'本文'.encode('utf-8'))

        message = django_mail.outbox[2].message()
        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example2@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")

    # FIXME
    @unittest.skip("Py3.6から動かない")
    def test_mass_mail_encoding_inline2(self):
        send_mass_mail((
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example0@example.net>'],
                'iso-2022-jp',
            ),
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example1@example.net>'],
            ),
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example2@example.net>'],
                'iso-2022-jp',
            )
        ), encoding='cp932')

        message = django_mail.outbox[0].message()
        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example0@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")

        message = django_mail.outbox[1].message()
        self.assertEqual(str(message['Subject']), "=?shift-jis?b?jI+WvA==?=")
        self.assertEqual(str(message['To']), "=?shift-jis?b?iLaQ5g==?= <example1@example.net>")
        self.assertEqual(str(message['From']),
                         "=?shift-jis?b?jbePb5Bs?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '8bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="shift-jis"')
        self.assertEqual(message.get_payload(), "\x96{\x95\xb6")

        message = django_mail.outbox[2].message()
        self.assertEqual(str(message['Subject']), "=?iso-2022-jp?b?GyRCN29MPhsoQg==?=")
        self.assertEqual(str(message['To']),
                         "=?iso-2022-jp?b?GyRCMDhAaBsoQg==?= <example2@example.net>")
        self.assertEqual(str(message['From']),
                         "=?iso-2022-jp?b?GyRCOjk9UD9NGyhC?= <example-from@example.net>")
        self.assertEqual(message['Content-Transfer-Encoding'], '7bit')
        self.assertEqual(message['Content-Type'], 'text/plain; charset="iso-2022-jp"')
        self.assertEqual(message.get_payload(), "\x1b$BK\\J8\x1b(B")

    def test_mass_mail_pre_send(self):
        test_list = []

        def pre_send_signal(sender, message, **kwargs):
            self.assertEqual(message.subject, u'件名')
            self.assertEqual(message.body, u'本文')
            test_list.extend(message.to)
        mail_pre_send.connect(pre_send_signal)

        send_mass_mail(((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)), encoding='iso-2022-jp')

        self.assertEqual(test_list, [
            u'宛先 <example0@example.net>',
            u'宛先 <example1@example.net>',
            u'宛先 <example2@example.net>',
            u'宛先 <example3@example.net>',
            u'宛先 <example4@example.net>',
            u'宛先 <example5@example.net>',
            u'宛先 <example6@example.net>',
            u'宛先 <example7@example.net>',
            u'宛先 <example8@example.net>',
            u'宛先 <example9@example.net>',
        ])

    def test_mass_mail_post_send(self):
        test_list = []

        def post_send_signal(sender, message, **kwargs):
            self.assertEqual(message.subject, u'件名')
            self.assertEqual(message.body, u'本文')
            test_list.extend(message.to)
        mail_post_send.connect(post_send_signal)

        send_mass_mail(((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)), encoding='iso-2022-jp')

        self.assertEqual(test_list, [
            u'宛先 <example0@example.net>',
            u'宛先 <example1@example.net>',
            u'宛先 <example2@example.net>',
            u'宛先 <example3@example.net>',
            u'宛先 <example4@example.net>',
            u'宛先 <example5@example.net>',
            u'宛先 <example6@example.net>',
            u'宛先 <example7@example.net>',
            u'宛先 <example8@example.net>',
            u'宛先 <example9@example.net>',
        ])

    def test_mass_mail_message_obj(self):
        send_mass_mail(chain((
            (
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example%s@example.net>' % i],
            ) for i in range(4)),
            (EmailMessage(u'件名', u'本文', u'差出人 <example-from@example.net>',
                          [u'宛先 <example4@example.net>']),),
            ((
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example%s@example.net>' % i],
            ) for i in range(5, 9)),
            (EmailMessage(u'件名', u'本文', u'差出人 <example-from@example.net>',
                          [u'宛先 <example9@example.net>']),)
        ))

        for i in range(10):
            message = django_mail.outbox[i].message()

            self.assertEqual(str(message['Subject']), "=?utf-8?b?5Lu25ZCN?=")
            self.assertEqual(str(message['To']),
                             "=?utf-8?b?5a6b5YWI?= <example%s@example.net>" % i)
            self.assertEqual(str(message['From']),
                             "=?utf-8?b?5beu5Ye65Lq6?= <example-from@example.net>")
            self.assertEqual(message['Content-Transfer-Encoding'], '8bit')
            self.assertEqual(message['Content-Type'], 'text/plain; charset="utf-8"')
            self.assertEqual(message.get_payload(decode=True), u'本文'.encode('utf-8'))


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf-8')
@override_settings(TIME_ZONE='Asia/Tokyo')
@override_settings(EMAIL_USE_LOCALTIME=False)
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class UTCTimeTestCase(MailTestCase, DjangoTestCase):

    def test_email_utc_strict(self):
        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()
        self.assertTrue(message['Date'].endswith("-0000"))


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf8')
@override_settings(TIME_ZONE='Asia/Tokyo')
@override_settings(EMAIL_USE_LOCALTIME=True)
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class LocalTimeTestCase(MailTestCase, DjangoTestCase):

    def test_email_localtime_strict(self):
        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
        )

        message = django_mail.outbox[0].message()
        self.assertTrue(message['Date'].endswith("+0900"))


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.tests.ErrorEmailBackend')
class FailSilentlyTestCase(MailTestCase, DjangoTestCase):

    def test_fail_silently(self):
        send_mail(
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            fail_silently=True,
        )
        send_template_mail(
            u'mailer/mail.tpl',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example@example.net>'],
            extra_context={
                'subject': u'件名',
                'body': u'本文',
            },
            fail_silently=True,
        )
        send_mass_mail(((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)), fail_silently=True)
        mail_managers(
            u'件名',
            u'本文',
            fail_silently=True,
        )
        mail_managers_template(
            u'mailer/mail.tpl',
            extra_context={
                'subject': u'件名',
                'body': u'本文',
            },
            fail_silently=True,
        )
        mail_admins(
            u'件名',
            u'本文',
            fail_silently=True,
        )

    def test_fail_loud(self):
        try:
            send_mail(
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example@example.net>'],
            )
            self.fail("Expected Error")
        except EmailError:
            pass
        try:
            send_template_mail(
                u'mailer/mail.tpl',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example@example.net>'],
                extra_context={
                    'subject': u'件名',
                    'body': u'本文',
                },
            )
            self.fail("Expected Error")
        except EmailError:
            pass
        try:
            send_mass_mail(((
                u'件名',
                u'本文',
                u'差出人 <example-from@example.net>',
                [u'宛先 <example%s@example.net>' % i],
            ) for i in range(10)))
            self.fail("Expected Error")
        except EmailError:
            pass
        try:
            mail_managers(
                u'件名',
                u'本文',
            )
            self.fail("Expected Error")
        except EmailError:
            pass
        try:
            mail_managers_template(
                u'mailer/mail.tpl',
                extra_context={
                    'subject': u'件名',
                    'body': u'本文',
                },
            )
            self.fail("Expected Error")
        except EmailError:
            pass
        try:
            mail_admins(
                u'件名',
                u'本文',
            )
            self.fail("Expected Error")
        except EmailError:
            pass


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class AttachmentTestCase(MailTestCase, DjangoTestCase):

    def test_send_mail(self):
        send_mail(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
            attachments=[('test.txt', u"データ", 'text/plain')],
        )

        message = django_mail.outbox[0]
        self.assertEquals(message.attachments, [('test.txt', u"データ", 'text/plain')])

    def test_send_template_mail(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'example-from@example.net',
            [u'example@example.net'],
            extra_context={
                'subject': u'件名',
                'body': u'本文',
                'html': u"<h1>本文</h1>",
            },
            fail_silently=False,
            html_template_name=u'mailer/html_mail.tpl',
            attachments=[('test.txt', u"データ", 'text/plain')],
        )

        message = django_mail.outbox[0]
        self.assertEquals(message.attachments, [('test.txt', u"データ", 'text/plain')])

    def test_binary_attachment(self):
        message = EmailMessage(
            attachments=[('test.binary', u"データ".encode("utf8"), None)]).message()

        payloads = message.get_payload()

        # 添付ファイルのペイロード
        self.assertEquals(len(payloads), 1)
        self.assertEquals(payloads[0]['Content-Transfer-Encoding'], 'base64')
        self.assertEquals(payloads[0]['Content-Type'], 'application/octet-stream')
        self.assertEquals(payloads[0]['Content-Disposition'], 'attachment; filename="test.binary"')
        self.assertEqual(payloads[0].get_payload(decode=True), u'データ'.encode('utf-8'))

    def test_text_attachment(self):
        message = EmailMessage(attachments=[('test.txt', u"データ", None)]).message()

        payloads = message.get_payload()

        # 添付ファイルのペイロード
        self.assertEquals(len(payloads), 1)
        self.assertEquals(payloads[0]['Content-Transfer-Encoding'], '8bit')
        self.assertEquals(payloads[0]['Content-Type'], 'text/plain; charset="utf-8"')
        self.assertEquals(payloads[0]['Content-Disposition'], 'attachment; filename="test.txt"')
        self.assertEqual(payloads[0].get_payload(decode=True), u'データ'.encode('utf-8'))


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class HtmlMailTestCase(MailTestCase, DjangoTestCase):

    def test_send_mail_html(self):
        send_mail(
            u"件名",
            u"本文",
            'example-from@example.net',
            ['example@example.net'],
            html_message=u"<h1>本文</h1>",
        )
        self.assertEquals(len(django_mail.outbox), 1)

        email_message = django_mail.outbox[0]
        self.assertEquals(email_message.body, u'本文')
        self.assertTrue(isinstance(email_message, EmailMultiAlternatives))

        self.assertTrue((u"<h1>本文</h1>", "text/html") in email_message.alternatives)

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')

        payloads = message.get_payload()

        # text + html ペイロード
        self.assertEquals(len(payloads), 2)
        self.assertEquals(payloads[0]['Content-Transfer-Encoding'], '8bit')
        self.assertEquals(payloads[0]['Content-Type'], 'text/plain; charset="utf-8"')
        self.assertEquals(payloads[0].get_payload(decode=True), u"本文".encode("utf-8"))
        self.assertEquals(payloads[1]['Content-Transfer-Encoding'], '8bit')
        self.assertEquals(payloads[1]['Content-Type'], 'text/html; charset="utf-8"')
        self.assertEquals(payloads[1].get_payload(decode=True), u"<h1>本文</h1>".encode("utf-8"))

    def test_html_template_mail(self):
        send_template_mail(
            u'mailer/mail.tpl',
            u'example-from@example.net',
            [u'example@example.net'],
            extra_context={
                'subject': u'件名',
                'body': u'本文',
                'html': u"<h1>本文</h1>",
            },
            fail_silently=False,
            html_template_name=u'mailer/html_mail.tpl',
        )

        self.assertEquals(len(django_mail.outbox), 1)

        email_message = django_mail.outbox[0]
        self.assertEquals(email_message.body, u'本文\n')
        self.assertTrue(isinstance(email_message, EmailMultiAlternatives))

        self.assertTrue((u"<h1>本文</h1>\n", "text/html") in email_message.alternatives)

        message = django_mail.outbox[0].message()
        self.assertEquals(str(message['Subject']), '=?utf-8?b?5Lu25ZCN?=')
        self.assertEquals(str(message['To']), 'example@example.net')
        self.assertEquals(str(message['From']), 'example-from@example.net')

        payloads = message.get_payload()

        # text + html ペイロード
        self.assertEquals(len(payloads), 2)
        self.assertEquals(payloads[0]['Content-Transfer-Encoding'], '8bit')
        self.assertEquals(payloads[0]['Content-Type'], 'text/plain; charset="utf-8"')
        self.assertEquals(payloads[0].get_payload(decode=True), u"本文\n".encode('utf-8'))
        self.assertEquals(payloads[1]['Content-Transfer-Encoding'], '8bit')
        self.assertEquals(payloads[1]['Content-Type'], 'text/html; charset="utf-8"')
        self.assertEquals(payloads[1].get_payload(decode=True), u"<h1>本文</h1>\n".encode('utf-8'))


@override_settings(ADMINS=(('Admin', 'admin@example.net'),))
@override_settings(MANAGERS=(('Manager', 'manager@example.net'),))
@override_settings(DEFAULT_CHARSET='utf8')
@override_settings(EMAIL_BACKEND='beproud.django.mailer.backends.locmem.EmailBackend')
class TaskTests(MailTestCase, DjangoTestCase):

    @mock.patch.object(mailer_api, 'send_mail')
    def test_send_mail(self, send_mail):
        mailer_tasks.send_mail.delay(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
        )

        send_mail.assert_called_once_with(
            u'件名',
            u'本文',
            'example-from@example.net',
            ['example@example.net'],
        )

    @mock.patch.object(mailer_api, 'send_template_mail')
    def test_send_template_mail(self, send_template_mail):
        mailer_tasks.send_template_mail.delay(
            u'mailer/mail.tpl',
            u'example-from@example.net',
            [u'example@example.net'],
            extra_context={
                'subject': u'件名',
                'body': u'本文',
                'html': u"<h1>本文</h1>",
            },
            fail_silently=False,
            html_template_name=u'mailer/html_mail.tpl',
        )

        send_template_mail.assert_called_once_with(
            u'mailer/mail.tpl',
            u'example-from@example.net',
            [u'example@example.net'],
            extra_context={
                'subject': u'件名',
                'body': u'本文',
                'html': u"<h1>本文</h1>",
            },
            fail_silently=False,
            html_template_name=u'mailer/html_mail.tpl',
        )

    @mock.patch.object(mailer_api, 'send_mass_mail')
    def test_send_mass_mail(self, send_mass_mail):
        mailer_tasks.send_mass_mail.delay(list((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)))

        send_mass_mail.assert_called_once_with(list((
            u'件名',
            u'本文',
            u'差出人 <example-from@example.net>',
            [u'宛先 <example%s@example.net>' % i],
        ) for i in range(10)))

    @mock.patch.object(mailer_api, 'mail_managers')
    def test_mail_managers(self, mail_managers):
        mailer_tasks.mail_managers.delay(
            u'件名',
            u'本文',
            fail_silently=True,
        )

        mail_managers.assert_called_once_with(
            u'件名',
            u'本文',
            fail_silently=True,
        )

    @mock.patch.object(mailer_api, 'mail_managers_template')
    def test_mail_managers_template(self, mail_managers_template):
        mailer_tasks.mail_managers_template.delay(
            u'mailer/mail.tpl',
            extra_context={
                'subject': u'件名',
                'body': u'本文',
            },
            fail_silently=True,
        )

        mail_managers_template.assert_called_once_with(
            u'mailer/mail.tpl',
            extra_context={
                'subject': u'件名',
                'body': u'本文',
            },
            fail_silently=True,
        )

    @mock.patch.object(mailer_api, 'mail_admins')
    def test_mail_admins(self, mail_admins):
        mailer_tasks.mail_admins.delay(
            u'件名',
            u'本文',
            fail_silently=True,
        )

        mail_admins.assert_called_once_with(
            u'件名',
            u'本文',
            fail_silently=True,
        )



"""
Email backend that writes messages to console instead of sending them.
"""
import sys
import logging
import threading

from django.conf import settings

from beproud.django.mailer.backends.base import BaseEmailBackend

logger = logging.getLogger(getattr(settings, "EMAIL_LOGGER", ""))


class EmailBackend(BaseEmailBackend):
    def __init__(self, *args, **kwargs):
        self.stream = kwargs.pop('stream', sys.stdout)
        self._lock = threading.RLock()
        super(EmailBackend, self).__init__(*args, **kwargs)

    def send_messages(self, email_messages):
        """Write all messages to the stream in a thread-safe way."""
        if not email_messages:
            return
        self._lock.acquire()
        num_sent = 0
        try:
            # The try-except is nested to allow for
            # Python 2.4 support (Refs #12147)
            stream_created = self.open()
            for message in email_messages:
                if self._send_message_wrapper(message):
                    num_sent += 1
            if stream_created:
                self.close()
        finally:
            self._lock.release()
        return num_sent

    def _send_message(self, email_message):
        self.stream.write('%s\n' % email_message.message().as_string())
        self.stream.write('-'*79)
        self.stream.write('\n')
        self.stream.flush()  # flush after each message



#:coding=utf-8:

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.text import normalize_newlines

register = template.Library()


def replace_newlines(text, replacement=' '):
    """
    Replaces all newline characters in a block of text.
    """
    normalized_text = normalize_newlines(text)
    return normalized_text.replace('\n', replacement)
replace_newlines.is_safe = True
replace_newlines = stringfilter(replace_newlines)
register.filter(replace_newlines)



#!/usr/bin/env python
#:coding=utf-8:

# XXX: Hack to prevent stupid "TypeError: 'NoneType' object is not callable" error
# in multiprocessing/util.py _exit_function when running `python
# setup.py test` (see
# http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing  # NOQA
except ImportError:
    pass

import sys
import os
import fnmatch

try:
    current_dir = os.path.dirname(os.path.abspath(__file__))
    for fn in os.listdir(current_dir):
        if fnmatch.fnmatch(fn, 'billiard-*.egg'):
            sys.path.append(os.path.join(current_dir, fn))

    import billiard  # NOQA
except ImportError:
    pass

############## End Hack ################

from setuptools import setup, find_packages

def read_file(filename):
    basepath = os.path.dirname(os.path.dirname(__file__))
    filepath = os.path.join(basepath, filename)
    if os.path.exists(filepath):
        with open(filepath) as f:
            read_text = f.read()
        return read_text
    else:
        return ''


setup(
    name='bpmailer',
    version='1.2',
    description='Mailing utility for Django',
    long_description=read_file('README.rst'),
    long_description_content_type="text/x-rst",
    author='BeProud Inc.',
    author_email='project@beproud.jp',
    url='https://github.com/beproud/bpmailer/',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'Environment :: Plugins',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    include_package_data=True,
    packages=find_packages(),
    namespace_packages=['beproud', 'beproud.django'],
    install_requires=['Django>=2.2', 'six', 'Celery'],
    test_suite='tests.main',
    zip_safe=False,
    keywords=['django', 'mail']
)



#:coding=utf-8:
try:
    from celery import shared_task
except ImportError:
    from celery.task import task as shared_task

from beproud.django.mailer import api as mailer_api

__all__ = (
    'send_mail',
    'send_template_mail',
    'send_mass_mail',
    'mail_managers',
    'mail_managers_template',
    'mail_admins',
)


@shared_task
def send_mail(*args, **kwargs):
    max_retries = kwargs.pop('max_retries', 3)
    retry_countdown = kwargs.pop('retry_countdown', 10)
    try:
        mailer_api.send_mail(*args, **kwargs)
    except Exception as e:
        return send_mail.retry(
            exc=e,
            countdown=retry_countdown,
            max_retries=max_retries,
        )


@shared_task
def send_template_mail(*args, **kwargs):
    max_retries = kwargs.pop('max_retries', 3)
    retry_countdown = kwargs.pop('retry_countdown', 10)
    try:
        mailer_api.send_template_mail(*args, **kwargs)
    except Exception as e:
        return send_template_mail.retry(
            exc=e,
            countdown=retry_countdown,
            max_retries=max_retries,
        )


@shared_task
def send_mass_mail(*args, **kwargs):
    max_retries = kwargs.pop('max_retries', 3)
    retry_countdown = kwargs.pop('retry_countdown', 10)
    try:
        mailer_api.send_mass_mail(*args, **kwargs)
    except Exception as e:
        return send_mass_mail.retry(
            exc=e,
            countdown=retry_countdown,
            max_retries=max_retries,
        )


@shared_task
def mail_managers(*args, **kwargs):
    max_retries = kwargs.pop('max_retries', 3)
    retry_countdown = kwargs.pop('retry_countdown', 10)
    try:
        mailer_api.mail_managers(*args, **kwargs)
    except Exception as e:
        return mail_managers.retry(
            exc=e,
            countdown=retry_countdown,
            max_retries=max_retries,
        )


@shared_task
def mail_managers_template(*args, **kwargs):
    max_retries = kwargs.pop('max_retries', 3)
    retry_countdown = kwargs.pop('retry_countdown', 10)
    try:
        mailer_api.mail_managers_template(*args, **kwargs)
    except Exception as e:
        return mail_managers_template.retry(
            exc=e,
            countdown=retry_countdown,
            max_retries=max_retries,
        )


@shared_task
def mail_admins(*args, **kwargs):
    max_retries = kwargs.pop('max_retries', 3)
    retry_countdown = kwargs.pop('retry_countdown', 10)
    try:
        mailer_api.mail_admins(*args, **kwargs)
    except Exception as e:
        return mail_admins.retry(
            exc=e,
            countdown=retry_countdown,
            max_retries=max_retries,
        )
